package net.pterodactylus.sone.data;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.equalTo;
import static org.hamcrest.Matchers.is;
import static org.hamcrest.Matchers.not;

import org.junit.Test;

/**
 * Unit test for {@link Id}.
 *
 * @author <a href="mailto:bombe@pterodactylus.net">David ‘Bombe’ Roden</a>
 */
public class IdTest {

	private static final String SONE_ID = "~Yp72VX0c6FLDvgIzip5wIvaGIIrjKcKvnX~pTaMKXs";
	private static final String ELEMENT_ID = "88CC70AE-E853-4EEE-B245-E4C55F40DDDF";
	private static final String EXPECTED_ID = "139a629a13f6a2c4191fb19ecead7e57335ea3deb2a971b88d5e004378c4daad";

	private final Id completeId = Id.from(SONE_ID, ELEMENT_ID);
	private final Id parsedId = Id.from(EXPECTED_ID);

	@Test
	public void idBuilderBuildsCorrectIds() {
		assertThat(completeId.getSoneId().get(), is(SONE_ID));
		assertThat(completeId.getElementId().get(), is(ELEMENT_ID));
		assertThat(completeId.getExternal(), is(EXPECTED_ID));
	}

	@Test
	public void parsedIdIsBuiltCorrectly() {
		assertThat(parsedId.getSoneId().isPresent(), is(false));
		assertThat(parsedId.getElementId().isPresent(), is(false));
		assertThat(parsedId.getExternal(), is(EXPECTED_ID));
	}

	@Test
	public void twoIdsAreNotEqualIfExternalIdIsDifferent() {
		Id id2 = Id.from(EXPECTED_ID + "1");
		assertThat(id2, not(equalTo(parsedId)));
		assertThat(parsedId, not(equalTo(id2)));
	}

	@Test
	public void twoIdsAreEqualIfSoneAndElementIdIsEqual() {
		Id id2 = Id.from(SONE_ID, ELEMENT_ID);
		assertThat(id2, equalTo(completeId));
		assertThat(completeId, equalTo(id2));
	}

	@Test
	public void twoIdsAreEqualIfExternalIdIsEqual() {
		Id id2 = Id.from(EXPECTED_ID);
		assertThat(id2, equalTo(parsedId));
		assertThat(parsedId, equalTo(id2));
	}

	@Test
	public void twoIdsHaveTheSameHashCodeIfTheyAreEqual() {
		Id id2 = Id.from(EXPECTED_ID);
		assertThat(id2.hashCode(), equalTo(completeId.hashCode()));
	}

	@Test
	public void nullIsNotEqualToId() {
		assertThat(completeId, not(equalTo(null)));
	}

	@Test
	public void objectOfDifferentClassIsNotEqualToId() {
		assertThat(completeId, not(equalTo(new Object())));
	}

	@Test
	public void stringRepresentationMatchesTheExternalId() {
		assertThat(completeId.toString(), is(EXPECTED_ID));
	}

	@Test(expected = NullPointerException.class)
	public void canNotParseNullAsExternalId() {
		Id.from(null);
	}

	@Test(expected = NullPointerException.class)
	public void canNotParseNullAsSoneId() {
		Id.from(null, ELEMENT_ID);
	}

	@Test(expected = NullPointerException.class)
	public void canNotParseNullAsElementId() {
		Id.from(SONE_ID, null);
	}

}
